"""Build lemma dictionary from bnkorpus."""

# ruff: noqa: T201

import sys
import zipfile
from pathlib import Path

from lxml import etree

from lemmatizer_be._utils import _fetch_unzip, dir_empty

DATA_DIR = Path(Path(__file__).parent.parent.parent, "data")
DATA_DIR.mkdir(parents=True, exist_ok=True)

BNKORPUS_DIR = Path(Path(__file__).parent.parent.parent, "bnkorpus")
BNKORPUS_DIR.mkdir(parents=True, exist_ok=True)

BNKORPUS_URL = "https://github.com/Belarus/GrammarDB/releases/download/RELEASE-202309/RELEASE-20230920.zip"


def strip_plus(word):  # noqa: D103
    return word.replace("+", "")


def main():  # noqa: D103
    print("bnkorpus status:", end=" ")

    if dir_empty(BNKORPUS_DIR):
        print("missing. Downloading...")
        _fetch_unzip(BNKORPUS_URL, BNKORPUS_DIR)
    else:
        print("OK")

    data_dict = {}

    for xml_path in BNKORPUS_DIR.glob("*.xml"):
        tree = etree.fromstring(xml_path.read_bytes())  # noqa: S320
        print(f"Loaded '{xml_path}'. Analyzing...", end=" ")
        sys.stdout.flush()

        for paradigm in tree.findall("Paradigm"):
            paradigm_lemma = strip_plus(paradigm.get("lemma"))

            for variant in paradigm.findall("Variant"):
                for form in variant.findall("Form"):
                    form_text = strip_plus(form.text)

                    if form_text not in data_dict:
                        data_dict[form_text] = set()

                    data_dict[form_text].add(paradigm_lemma)

        print("OK")

    changeable = {}
    leaveable = []

    for k, v in data_dict.items():
        list_v = list(v)

        if len(list_v) == 1 and k == list_v[0]:
            leaveable.append(k)
        else:
            changeable[k] = list_v

    print(
        f"Found {len(leaveable):_} words to be left unchanged and {len(changeable):_} changeable words"
    )

    # region Writing data
    changeable_file_path = DATA_DIR / "change.tsv"

    with open(changeable_file_path, "w", encoding="utf8") as f:
        for word, lemmas in changeable.items():
            f.write("{}\t{}\n".format(word, ";".join(lemmas)))

    print(
        f"The changeable file size is {changeable_file_path.stat().st_size / 1024 / 1024:2f} MB"
    )

    leaveable_file_path = DATA_DIR / "leave.txt"

    with open(leaveable_file_path, "w", encoding="utf8") as f:
        for word in leaveable:
            f.write(word)
            f.write("\n")

    print(
        f"The leaveable file size is {leaveable_file_path.stat().st_size / 1024 / 1024:2f} MB"
    )
    # endregion

    # region Compressing
    arc_path = DATA_DIR / "lemma_data.zip"

    with zipfile.ZipFile(
        str(arc_path.resolve()),
        mode="w",
        compression=zipfile.ZIP_DEFLATED,
        compresslevel=6,
    ) as zip_file:
        zip_file.write(str(changeable_file_path.resolve()), "change.json")
        zip_file.write(str(leaveable_file_path.resolve()), "leave.txt")

    print(f"The arc file size is {arc_path.stat().st_size / 1024 / 1024:2f} MB")
    # endregion


if __name__ == "__main__":
    main()



import pytest

from lemmatizer_be.lemmatizer import BnkorpusLemmatizer

lemmatizer = BnkorpusLemmatizer()

word_lemma_list = (
    ("амлету", ["амлет"]),
    ("в", ["в"]),
    ("робіш", ["рабіць"]),
    (
        "касы",
        ["каса", "касы"],
    ),  # Каса - валасы або каса з грашыма, касы - гэта чалавек з дадатковымі патрэбамі
    ("мех", ["мех"]),
)


@pytest.mark.parametrize(("word", "lemmas"), word_lemma_list)
def test_word_lemmas(word, lemmas):
    assert set(lemmatizer.lemmas(word)) == set(lemmas)


word_lemma = (
    ("перапісваеш", "перапісваць"),
    ("бівака", "бівак"),
    ("абкружанаму", "акружаны"),
    ("Ляхавічаў", "Ляхавічы"),
    ("хрумшчу", "хрумсцець"),
)


@pytest.mark.parametrize(("word", "lemma"), word_lemma)
def test_word_lemma(word, lemma):
    assert lemmatizer.lemmatize(word) == lemma



"""The lemmatizer main file."""

# ruff: noqa: T201

from __future__ import annotations

import csv
from pathlib import Path

from lemmatizer_be._utils import _fetch_unzip, dir_empty

DATA_DIR = Path(Path(__file__).parent.parent.parent, "data")
DATA_DIR.mkdir(parents=True, exist_ok=True)

LEMMA_DATA_URL = "https://github.com/alex-rusakevich/lemmatizer-be/releases/latest/download/lemma_data.zip"


class BnkorpusLemmatizer:
    """Belarusian language lemmatizer based on bnkorpus."""

    def __init__(self):
        """Load the lemma dictionaries into memory."""
        if dir_empty(DATA_DIR):
            print("The lemmatizer's data is missing, downloading...")
            _fetch_unzip(LEMMA_DATA_URL, DATA_DIR)
            print("The lemmatizer's data has been downloaded successfully.")

        self._changeable = {}

        with open(DATA_DIR / "change.tsv", "r", encoding="utf8") as f:
            tsv_file = csv.reader(f, delimiter="\t")

            for line in tsv_file:
                self._changeable[line[0]] = line[1].split(";")

        self._unchangeable = (DATA_DIR / "leave.txt").read_text(encoding="utf8").split()

    def lemmas(self, word: str) -> list[str]:
        """Return list of all the lemmas for the word.

        Parameters
        ----------
        word : str
            the word lemmatizer finds lemmas for

        Returns
        -------
        list[str]
            list of lemmas if any

        """
        if word in self._unchangeable:
            return [word]

        lemma = self._changeable.get(word, None)

        return lemma

    def lemmatize(self, word: str) -> str:
        """Lemmatize ``word`` by picking the shortest of the possible lemmas.

        Uses ``self.lemmas()`` internally.
        Returns the input word unchanged if it cannot be found in WordNet.

        Parameters
        ----------
        word : str
            the word lemmatizer finds lemma for

        Returns
        -------
        str
            the lemma found by lemmatizer

        """
        lemmas = self.lemmas(word)
        return min(lemmas, key=len) if lemmas else word



from __future__ import annotations

import io
import os
import zipfile
from pathlib import Path

import requests
from tqdm import tqdm


def dir_empty(dir_path: str | Path) -> bool:
    return not any(True for _ in os.scandir(dir_path))


def _fetch_unzip(zip_file_url: str, destination_dir: Path | str) -> Path:
    Path(destination_dir).mkdir(exist_ok=True, parents=True)
    bio = io.BytesIO()

    response = requests.get(zip_file_url, stream=True, timeout=10)
    with tqdm.wrapattr(
        bio,
        "write",
        miniters=1,
        desc=zip_file_url.split("/")[-1],
        total=int(response.headers.get("content-length", 0)),
    ) as fout:
        for chunk in response.iter_content(chunk_size=4096):
            fout.write(chunk)

    z = zipfile.ZipFile(bio)
    z.extractall(destination_dir)

    return Path(destination_dir)
